import math

iusch = 8
iusrc = 9
iusta = 10
iupk = 25
ilog = 29
unset = -9999.
onesec = 1./86400.
secpday = 86400
raddeg = math.pi / 180.
radds = math.pi / 180. / 3600.
radhs = math.pi / 12. / 3600.
radhr = math.pi / 12

max_seg = 100 # MSEG from sched.inc
sidereal = 1.00273791
