      SUBROUTINE OPENWRAP ( UNIT, RNAME, RTYPE, RSTATUS, RESULT, RET)
Cf2py intent(in) UNIT, RNAME, RTYPE, RSTATUS
Cf2py intent(out) RESULT, RET
      INTEGER UNIT
      CHARACTER RNAME*80
      CHARACTER RTYPE*10
      CHARACTER RSTATUS*10
      CHARACTER RESULT*255
      INTEGER RET, VLBOPE
      RET = VLBOPE(UNIT, RNAME, RTYPE, RSTATUS, RESULT)
      RETURN
      END
