      SUBROUTINE VSOPWRT
C
C     Stub for VSOP writing stuff in use in Japan.
C
C --------------------------------------------------------------------
      CALL WLOG( 1, 'VSOPWRT: Don''t have the VSOP routines.' )
      RETURN
      END
